/*
 *	GraphInit - Initialize TARGA 16 to default values
 *
 *	Usage:    int  ret;
 *			  ret = GraphInit(value);
 *
 *			  where:
 *				value =  8  if you explicitly want a TARGA 8
 *						16  if you explicitly want a TARGA 16
 *					    24  if you explicitly want a TARGA 24
 *					    32  if you explicitly want a TARGA 32
 *						-1  Use board type specified by TARGASET
 *					        or board during autoseach
 *							(Actually any other value)
 *
 *				ret== 0 if the I/O space is correctly found 
 *				ret== -1 if the I/O space is not correctly found.
 *						 This may result from incorrectly setting
 *						 the configuration switches or by incorrectly
 *						 setting the MS-DOS variable, TARGA.
 *
 */
#include "tardev.h"
struct  TARStruct tstructure;
struct  TARStruct *targa;
struct  M8Struct  mstructure;
struct  M8Struct  *m8;
/*   #define DEBUG 1    DEFINE TO INCLUDE FOR BOARD CHECKING */

GraphInit(board)
int board;
{
int bottom, top;
int  mode, tBoard;
	targa = &tstructure;
	m8    = &mstructure;

	/*
	 *	Set Defaults for the TARGASET Variable:   Do this initialize
	 *  in explicit statements rather than in "static" definition
	 *  so that you easily reinitialize the board despite everything
	 *  you may have done to it by calling GraphInit again.
	 */
	targa->boardType = -1;		/*  assume the resident board */
	targa->xOffset = 0;  targa->yOffset = 0;  /* default to no offset */
	targa->LinesPerField= DEF_ROWS/2; /*  number of lines per field */
	targa->AlwaysGenLock = DEF_GENLOCK;	/* default to genlock off */
	targa->PageMode = 0;
	targa->InterlaceMode =DEF_INT;   /* Defalut:  Interlace Mode 0 */
	targa->Contrast= DEF_CONTRAST;
	targa->Saturation = DEF_SATURATION;
	targa->Hue = DEF_HUE;
	targa->RGBorCV = CV;		/*  default to Composite video */
	targa->VCRorCamera = CAMERA;
	targa->PanXOrig = 0; targa->PanYOrig=0;
	targa->PageUpper= 0xffff;	/*  set the bank flags to illega& values */
	targa->PageLower= 0xffff;   /*  so that they will be set the first time */
	targa->ovrscnAvail=0;       /*  Assume no Overscan option */
	/*
	 *		Set defaults for the M8 structure
	 */
	m8->there     = 0;   /*  default to not present */
	m8->inOffset  = DEF_M8OFFSET;    
	m8->inGain    = DEF_M8GAIN;
	m8->inMap     = DEF_M8INMAP;
	m8->outMap    = DEF_M8OUTMAP;
	m8->inputMux  = DEF_M8MUX;
	m8->loopThru  = 0;
	 m8->modeRegister = DEF_M8MODE;
	 m8->CGAInterlace = DEF_CGAINTERLACE;

	
	/*
	 *		Get default values from the DOS variable -- TARGASET
	 */ 
	 tBoard = board;
	TargaSettings( 	&board, &targa->xOffset,
			&targa->yOffset,  &targa->LinesPerField,
			&targa->AlwaysGenLock,  &targa->InterlaceMode,
			&targa->Contrast, &targa->Saturation,
			&targa->Hue, &targa->RGBorCV , &targa->VCRorCamera,
			&targa->ovrscnAvail, &m8->inOffset, &m8->inGain,
			&m8->loopThru,  &m8->CGAInterlace);

	 	targa->ovrscnOn = targa->ovrscnAvail;

	if ( (tBoard == TYPE_8 )||(tBoard==TYPE_16)||
	     (tBoard == TYPE_24)||(tBoard==TYPE_32)||(tBoard==TYPE_M8) )
											board = tBoard;



	 /*
	  *			see if we specify the TARGA enviromental variable or not
	  *	
	  *	Set the default values for IO Address, and Memory Segment
	  */
	targa->iobase = DEF_IOBASE;
	targa->memloc = DEF_MEMSEG;


	/*
	 *	Parse the enviromental variable TARGA (if defined)
	 *
	 *	If not, use the defaults that we just set or the values that
	 *	we auto-search for.
	 *
	 *  Note:  If you specify the TARGA variable--- You must specify
	 *		   the targa type in TARGASET
	 */
	if ( TargaJumpers(&targa->iobase, &targa->memloc) == 0 )
						FindTarga(&board,&targa->iobase,&targa->memloc);
	else  {
		if ( board ==  -1  )
					board = FindBoardType(targa->iobase, targa->memloc);
	}
	/*
	 *				look for specified board type in TARGASET
	 *				FindTarga will attempt to find board of correct
	 *				type
	 */
	if ( (board == TYPE_8 )||(board==TYPE_16)||
	     (board == TYPE_24)||(board==TYPE_32)||(board==TYPE_M8) )
											targa->boardType = board;
    else	targa->boardType = TYPE_16;

#ifdef DEBUG
	printf("\nBOARD %d  targa-boardType %d ", board, targa->boardType);
#endif

	
	/*
	 *		Set flag if M8 present
	 */
	m8->there =  ( targa->boardType == TYPE_M8 )  ;

	if ( (targa->boardType == TYPE_8) || m8->there ) {
			targa->MaxBanks      = 8;
			targa->BytesPerPixel = 1;
    }
	if ( targa->boardType == TYPE_16 ) {
			targa->MaxBanks = 16;
			targa->BytesPerPixel = 2;
    }
	if ( targa->boardType == TYPE_24 ) {
			targa->MaxBanks = 32;
			targa->BytesPerPixel = 3;
    }
	if ( targa->boardType == TYPE_32 ) {
			targa->MaxBanks = 32;
			targa->BytesPerPixel = 4;
    }


	/*						Compute # of rows per 32K bank  */
	targa->RowsPerBank  =  512/(targa->MaxBanks);


	/*				divide Maxbanks by 16 to get the amount
	 *				to shift the addresses by
	 *				TARGA-8 and M8   ----   0
	 *				TARGA-16  ----   1
	 *				TARGA 24/32 ---  2
	 */
	targa->AddressShift =  targa->MaxBanks>>4;



	/*
	 *		if initializing CVA:  set these before we quit
	 */
	SetSaturation(targa->Saturation);
	SetHue(targa->Hue);
	SetContrast( targa->Contrast);
	
	/*					Set Genlock bit if always genlocked */
	/*					set before flipping and jerking screen */
	SetMode( (targa->AlwaysGenLock<<SHF_GENLOCK) | DEF_MODE);


	SetBlndReg(0);		/*  disable blend mode on TARGA 32 */
						/*  internal logic - SetBlndReg    */
						/*   makes this a no op for the 8, M8, 16, and 24 */


	/*
	 *		Set Interlace Mode
	 */
	SetInterlace(targa->InterlaceMode);
	/*
	 *	Set Fixed Display Registers
	 */
	SetFixedRegisters();
	/*
	 *	Compute X border 
	 */
	 SetOverscan(targa->ovrscnOn);

	/*
	 *	Set the Vertical Border
	 */
	 top = 140 - (targa->LinesPerField>>1);
	 bottom =   top+targa->LinesPerField;
	/*
	 *	Ensure that YOffset does not blow us out of the screen-  
	 *   with number of lines specified by Page Length
	 */
	if ( targa->yOffset > 0 ) {
		if ( (top -= targa->yOffset) < MIN_TOP ) {
			targa->yOffset = MIN_TOP-top;
			top = MIN_TOP;
		}	
		bottom -= targa->yOffset;
	}
	if ( targa->yOffset < 0 ) {
		if ( (bottom -= targa->yOffset) > MAX_BOTTOM ) {
			targa->yOffset = MAX_BOTTOM-bottom;
			bottom = MAX_BOTTOM;
		}
		top -= targa->yOffset;
	}
	SetVBorder(top,bottom);



	if ( m8->there ) {
		SetVCRorCamera(targa->VCRorCamera);
		SetM8Offset(m8->inOffset);
		SetM8Gain  (m8->inGain);
		SetM8InMap (m8->inMap);
		SetM8OutMap(m8->outMap);
		SetM8Mux   (m8->inputMux);
	}

/*
 *	Set other Control Registers
 */
	SetMask(DEF_MASK);
	SetRGBorCV (targa->RGBorCV );
	SetVCRorCamera(targa->VCRorCamera);
	SetVertShift(256-targa->LinesPerField);

	/*
	 *		See if the raster register is working correctly and
	 *      return error flag if its not
	 */
	 if ( VWait()== -1) return(-1);
	 else				return(0);
}
extern     char *envfind();
/*
 *	TargaJumpers - parse the jumpers environment variable
 *
 *	Returns 0 if TARGA is not defined
 *  Returns 1 i TARGA is defined.
 *	
 */

extern  char *GETENV( char * );

TargaJumpers( ioBase, memLoc)
int	*ioBase;	/* pointer to I/O base value		*/
int	*memLoc;	/* pointer to memory location value	*/
{
	char *saveJump, *jumpers;  /* pointer to jumper string */

	jumpers = GETENV("TARGA");    /* get jumper state		*/
	saveJump = jumpers;

	
	if(jumpers == 0) return(0);	/*  return if string not defined */

		*ioBase =0x200;
		*memLoc = 0x8000;
	do {
		/*
		 *	Parse Switch 2-4 for memory location
		 */
		if ( (*jumpers>='2') && (*jumpers<='4') ) {
			*memLoc += 0x1000<<((int)('4'-*jumpers));
		}	
		/*	
		 *	Parse Switches 5-8 for the I/O location
		 */
		if ( (*jumpers>='5') && (*jumpers<='8') ) {
			*ioBase += 0x010<<((int)('8'-*jumpers));
		}	
	} while (*++jumpers !='\0');	
	return(1);
}
GetNumber(string,next)
char *string;
char  *(*next);
{
	char  *save, temp[30];

	save = string;

	strcpy(temp,string);

	while (  (*save==' ') || (*save=='\t') ||  (*save=='-') ) save++;

	while ( (*save>='0') && (*save<= '9') ) save++;

	*next = save;
	temp[save-string] = '\0';

	return(atoi(temp));
}

/*
 *	TargaSettings - parse the settings environment variable TARGASET
 *
 */

TargaSettings( boardType, xOffset,    yOffset, LinesPerField,
		AlwaysGenLock,  InterlaceMode,   Contrast, Saturation, Hue,
		RGBorCV , VCRorCamera, overScan, m8Offset, m8Gain, m8LoopThru,
		m8CGAInterlace)
int *boardType;		/*  T16 T32 T24 or T8 */
int *xOffset;		/*  Desired X-Offset */
int *yOffset;		/*  Desired Y-Offset */
int *LinesPerField;    /*  Desired number of lines per Screen */
int *AlwaysGenLock;
int *InterlaceMode;	/*  Desired Interlace Mode */
int *Contrast;
int *Saturation;
int *Hue;
int  *RGBorCV ;	/*  CV or RGB */
int  *VCRorCamera;	/*  CV or RGB */
int  *overScan;     /*  overscan option or not */
int  *m8Offset;     /*   offset for the M8 board */
int  *m8Gain;       /*   gain  for the M8 board */
int   *m8LoopThru;  /*   Loop Through for the M8 board */
int   *m8CGAInterlace;  /*  CGA Interlace Mode */
{
	register char	*curCPtr;	/* current character pointer	*/
	char	*nextCPtr;		/* next character pointer	*/
	int	number;			/* converted number		*/
	char    *setvar;


	setvar = GETENV("TARGASET");	/* get icb settings		*/
	if(setvar == 0) return;

	curCPtr = setvar;

	while(*curCPtr != '\0' ){
		switch(*curCPtr){
		case 'T':
		case 't':
			curCPtr++;
			number = GetNumber(curCPtr, &nextCPtr);
			if(nextCPtr != curCPtr){ /* possible number	*/
				curCPtr = nextCPtr;
				if ( (number==8) || (number==16) ||
				     (number==24)|| (number==32))
					*boardType = number;
			}
			break;

		case 'X':
		case 'x':
			curCPtr++;
			number = GetNumber(curCPtr,&nextCPtr);
			if(nextCPtr != curCPtr){  /* possible number	*/
				curCPtr = nextCPtr;
				if ( number > 10 ) number =  10;
				if ( number <-10 ) number = -10;
				*xOffset = (int)number;
			}
			break;

		case 'Y':
		case 'y':
			curCPtr++;
			number = GetNumber(curCPtr, &nextCPtr);
			if(nextCPtr != curCPtr){ /* possible number	*/
				curCPtr = nextCPtr;
				*yOffset = (int)number;
			}
			break;

		case 'C':
		case 'c':
			curCPtr++;
			number = GetNumber(curCPtr, &nextCPtr);
			if(nextCPtr != curCPtr){	/* possible number	*/
				curCPtr = nextCPtr;
				if(number <= 100 && number >= 0){
					*Contrast =
						(MAX_CONTRAST*(int)number)/100;
				}
			}
			break;

		case 'S':
		case 's':
			curCPtr++;
			number = GetNumber(curCPtr, &nextCPtr);
			if(nextCPtr != curCPtr){ /* possible number	*/
				curCPtr = nextCPtr;
				if(number <= 100 && number >= 0){
					*Saturation =
					(MAX_SATURATION*(int)number)/100;
				}
			}
			break;

		case 'H':
		case 'h':
			curCPtr++;
			number = GetNumber(curCPtr, &nextCPtr);
			if(nextCPtr != curCPtr){	/* possible number	*/
				curCPtr = nextCPtr;
				if(number <= 100 && number >= 0){
					*Hue = (MAX_HUE*(int)number)/100;
				}
			}
			break;

		case 'p':
		case 'P':
			curCPtr++;
			number = GetNumber(curCPtr, &nextCPtr);
			if(nextCPtr != curCPtr){ /* possible number	*/
				curCPtr = nextCPtr;
				if(number >= 242 ) number = 241;
				if(number <= 0 ) number= 200;
				*LinesPerField= number;
			}
			break;

		case 'I':
		case 'i':
			*InterlaceMode=0;
			curCPtr++;
			number = GetNumber(curCPtr, &nextCPtr);
			if(nextCPtr != curCPtr){ /* possible number	*/
				curCPtr = nextCPtr;
				if( (number <= 3) && (number>=0) )
				 	*InterlaceMode = number;
			}
			break;

		case 'N':
		case 'n':
			*InterlaceMode=2;
			curCPtr++;
			break;

		case 'E':
		case 'e':
			*overScan=1;
			curCPtr++;
			break;

		case 'G':
		case 'g':
			*AlwaysGenLock=1;
			curCPtr++;
			break;

		case 'R':
		case 'r':
			*RGBorCV =RGB;
			curCPtr++;
			break;

		case 'B':
		case 'b':
			*VCRorCamera=CAMERA;
			curCPtr++;
			break;

		case 'V':
		case 'v':
			*VCRorCamera=VCR;
			curCPtr++;
			break;

		case 'o':
		case 'O':
			curCPtr++;
			number = GetNumber(curCPtr, &nextCPtr);
			if(nextCPtr != curCPtr){ /* possible number	*/
				curCPtr = nextCPtr;
				if(number >= 255 ) number = 255;
				if(number <= 0 ) number= 0;
				*m8Offset= number;
			}
			break;

		case 'a':
		case 'A':
			curCPtr++;
			number = GetNumber(curCPtr, &nextCPtr);
			if(nextCPtr != curCPtr){ /* possible number	*/
				curCPtr = nextCPtr;
				if(number >= 255 ) number = 255;
				if(number <= 0 ) number= 0;
				*m8Gain= number;
			}
			break;

		case 'L':
		case 'l':
			*m8LoopThru=1;
			curCPtr++;
			number = GetNumber(curCPtr, &nextCPtr);
			if(nextCPtr != curCPtr){ /* possible number	*/
				curCPtr = nextCPtr;
				if(number >= 3 ) number = 3;
				if(number <= 0 ) number= 0;
				*m8CGAInterlace= number;
			}
			break;


		default:
			curCPtr++;	/* skip unknown character	*/
		}
	}
	return;
}

#define	FIXED_REGS	10
static	int FixedRegs[]=
	{DR6,DR7,DR12,DR13,DR14,DR15,DR16,DR17,DR18,DR19};
static  int FixedValue[]=
	{DEF_DR6,DEF_DR7,DEF_DR12,DEF_DR13,DEF_DR14,
	 DEF_DR15,DEF_DR16,DEF_DR17,DEF_DR18,DEF_DR19};
/*
 *	SetFixedRegisters()
 *
 *	Set the registers which have required values
 */
SetFixedRegisters()
{
int reg;

	for ( reg=0; reg<FIXED_REGS; reg++)
		SetDispReg(FixedRegs[reg],FixedValue[reg]);
	return;
}
